# LLM Node.js Toolkit

一个用于简化大模型应用开发的 Node.js 工具包，集成了 LangChain、RAG、Agent 等功能，帮助开发者轻松构建基于大模型的应用。

## 特性

- 🔍 **RAG (检索增强生成)** - 轻松构建基于文档的问答系统
- 🤖 **Agent** - 使用工具增强的智能代理
- 💬 **聊天** - 简化聊天机器人应用构建
- 🛠️ **工具集** - 通用工具函数集合
- 🔄 **模型检测与管理** - 检测环境中可用的模型并执行健康检查
- 📝 **提示模板构建** - 轻松创建和管理提示模板

## 安装

```bash
# 使用npm
npm install llm-nodejs-toolkit

# 使用pnpm
pnpm add llm-nodejs-toolkit

# 使用yarn
yarn add llm-nodejs-toolkit
```

## 快速开始

### RAG 示例

以下是一个基本的 RAG 应用实现：

```typescript
import {
  ModelProvider,
  RAGBuilder,
  createAndSplitDocument,
} from "llm-nodejs-toolkit";

async function main() {
  // 准备文档
  const docs = await createAndSplitDocument(
    "人工智能（Artificial Intelligence，简称AI）是计算机科学的一个分支，致力于创造能够模拟人类智能的机器。" +
      "大语言模型（Large Language Model，简称LLM）是一种基于深度学习的AI系统，经过大规模文本训练，" +
      "能够生成类似人类的自然语言文本。像GPT、LLaMA、Claude等都是知名的大语言模型。",
    { source: "人工智能简介" }
  );

  // 创建RAG系统
  const rag = await new RAGBuilder()
    .addDocuments(docs)
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: "your-api-key",
    })
    .withRAGConfig({
      embeddingModelName: "text-embedding-ada-002",
      vectorStoreType: "memory",
    })
    .build();

  // 查询RAG系统
  const answer = await rag.query("什么是大语言模型？");
  console.log(answer);
}

main().catch(console.error);
```

### Agent 示例

```typescript
import {
  ModelProvider,
  AgentBuilder,
  createCalculatorTool,
  createDateTimeTool,
} from "llm-nodejs-toolkit";

async function main() {
  // 创建Agent
  const agent = await new AgentBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: "your-api-key",
    })
    .withTool(createCalculatorTool())
    .withTool(createDateTimeTool())
    .build();

  // 执行查询
  const result = await agent.execute("今天是几号？并计算25乘以16等于多少？");
  console.log(result.output);
}

main().catch(console.error);
```

### 聊天示例

```typescript
import { ModelProvider, ChatBuilder } from "llm-nodejs-toolkit";

async function main() {
  // 创建聊天应用
  const chat = new ChatBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: "your-api-key",
    })
    .withSystemPrompt("你是一个友好的AI助手，擅长用简洁和友好的方式回答问题。")
    .withConfig({
      memoryMessageLimit: 10,
      streamingEnabled: true,
    })
    .build();

  // 发送消息
  const response1 = await chat.sendMessage("你好，请介绍一下自己。");
  console.log(response1);

  // 发送第二条消息（会记住上下文）
  const response2 = await chat.sendMessage("你能做什么？");
  console.log(response2);
}

main().catch(console.error);
```

### 新增功能：模型检测与健康检查

```typescript
import {
  ModelProvider,
  detectAvailableModels,
  checkLLMHealth,
} from "llm-nodejs-toolkit";

async function main() {
  // 检测可用的模型
  const availableModels = await detectAvailableModels([ModelProvider.OPENAI], {
    [ModelProvider.OPENAI]: "your-api-key",
  });

  console.log("可用的模型:", availableModels);

  // 检查模型健康状况
  const healthResult = await checkLLMHealth({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: "your-api-key",
  });

  if (healthResult.success) {
    console.log("模型健康检查通过!");
    console.log(`响应延迟: ${healthResult.latencyMs}ms`);
  } else {
    console.error("模型健康检查失败:", healthResult.error);
    console.log("建议:", healthResult.suggestion);
  }
}

main().catch(console.error);
```

### 新增功能：提示模板构建

```typescript
import { PromptBuilder, createRAGPromptTemplate } from "llm-nodejs-toolkit";
import { ChatOpenAI } from "@langchain/openai";

async function main() {
  const llm = new ChatOpenAI({
    openAIApiKey: "your-api-key",
    modelName: "gpt-3.5-turbo",
  });

  // 方法1: 使用PromptBuilder构建自定义提示
  const customPrompt = new PromptBuilder()
    .addSystemMessage("你是一个营销专家，擅长创建吸引人的广告文案。")
    .addExampleDialog(
      "为一款新款运动鞋写一段简短的广告语",
      "突破极限，释放潜能。全新设计，只为让你的每一步都充满力量。"
    )
    .addHumanMessage("为{product}创建一段广告文案")
    .build();

  const result1 = await customPrompt.pipe(llm).invoke({ product: "智能手表" });
  console.log(result1);

  // 方法2: 使用预定义模板
  const ragPrompt = createRAGPromptTemplate();

  const result2 = await ragPrompt.pipe(llm).invoke({
    context: "苹果公司是一家美国跨国科技公司，总部位于加利福尼亚州。",
    question: "苹果公司总部在哪里？",
  });

  console.log(result2);
}

main().catch(console.error);
```

## 关于 RAG

检索增强生成(RAG)是一种结合了检索系统和生成式 AI 的技术，它可以:

1. 从大型文档库中检索相关信息
2. 将检索到的信息作为上下文提供给大语言模型
3. 生成基于事实和最新信息的回答

RAG 特别适合需要根据特定文档或知识库回答问题的场景，如客服机器人、产品助手等。

## 关于 Agent

Agent 是自主行动的 AI 系统，它可以:

1. 使用多种工具完成复杂任务
2. 自主决定何时使用哪个工具
3. 根据工具返回的结果制定下一步计划
4. 与用户进行多轮交互

Agent 适合需要多步骤推理和工具使用的场景，如任务规划、计算分析等。

## 更多示例

查看 `examples` 目录获取更多使用示例：

- `chat-example.ts`: 聊天功能示例
- `rag-example.ts`: 检索增强生成示例
- `agent-example.ts`: 智能代理示例
- `model-health-check-example.ts`: 模型检测和健康检查示例
- `prompt-builder-example.ts`: 提示模板构建示例

## 许可证

MIT
