/**
 * Agent构建器模块
 *
 * 用于简化Agent的构建过程
 */
import { ChatPromptTemplate } from "@langchain/core/prompts";
import { BaseMessage } from "@langchain/core/messages";
import { DynamicStructuredTool } from "@langchain/core/tools";
import { LLMConfig } from "../types.js";
/**
 * Agent构建器类
 *
 * 用于简化LLM Agent的构建过程
 */
export declare class AgentBuilder {
    private llmConfig?;
    private tools;
    private executionHistory;
    private prompt?;
    private agentExecutor?;
    /**
     * 配置Agent的语言模型
     * @param config LLM配置
     * @returns 当前Agent构建器实例
     */
    withLLM(config: LLMConfig): AgentBuilder;
    /**
     * 添加工具到Agent
     * @param tools 工具数组
     * @returns 当前Agent构建器实例
     */
    withTools(tools: DynamicStructuredTool[]): AgentBuilder;
    /**
     * 添加单个工具到Agent
     * @param tool 工具
     * @returns 当前Agent构建器实例
     */
    withTool(tool: DynamicStructuredTool): AgentBuilder;
    /**
     * 添加执行历史记录，用于支持多轮对话
     * @param messages 历史消息数组
     * @returns 当前Agent构建器实例
     */
    withExecutionHistory(messages: BaseMessage[]): AgentBuilder;
    /**
     * 自定义Agent提示模板
     * @param prompt 自定义prompt模板
     * @returns 当前Agent构建器实例
     */
    withCustomPrompt(prompt: ChatPromptTemplate): AgentBuilder;
    /**
     * 构建默认的Agent提示模板
     * @returns 提示模板
     */
    private buildDefaultPrompt;
    /**
     * 构建Agent
     * @returns 当前Agent构建器实例
     */
    build(): Promise<AgentBuilder>;
    /**
     * 执行Agent查询
     * @param input 用户输入
     * @returns 执行结果
     */
    execute(input: string): Promise<{
        output: string;
        intermediateSteps: any[];
    }>;
}
