/**
 * 聊天构建器模块
 *
 * 用于简化聊天应用的构建过程
 */
import { BaseMessage } from "@langchain/core/messages";
import { LLMConfig } from "../types.js";
/**
 * 聊天配置接口
 */
export interface ChatConfig {
    /** 系统提示文本 */
    systemPrompt?: string;
    /** 记忆消息的最大数量 */
    memoryMessageLimit?: number;
    /** 记忆消息的最大token数量 */
    memoryTokenLimit?: number;
    /** 流式响应开关 */
    streamingEnabled?: boolean;
}
/**
 * 聊天构建器类
 *
 * 用于构建聊天应用
 */
export declare class ChatBuilder {
    private llmConfig?;
    private chatConfig;
    private chatHistory;
    private llm?;
    private chain?;
    /**
     * 配置聊天的语言模型
     * @param config LLM配置
     * @returns 当前聊天构建器实例
     */
    withLLM(config: LLMConfig): ChatBuilder;
    /**
     * 配置聊天参数
     * @param config 聊天配置
     * @returns 当前聊天构建器实例
     */
    withConfig(config: Partial<ChatConfig>): ChatBuilder;
    /**
     * 添加聊天历史
     * @param messages 聊天消息数组
     * @returns 当前聊天构建器实例
     */
    withHistory(messages: BaseMessage[]): ChatBuilder;
    /**
     * 设置系统提示
     * @param systemPrompt 系统提示文本
     * @returns 当前聊天构建器实例
     */
    withSystemPrompt(systemPrompt: string): ChatBuilder;
    /**
     * 构建聊天应用
     * @returns 当前聊天构建器实例
     */
    build(): ChatBuilder;
    /**
     * 修剪历史记录以符合token限制
     * @private
     */
    private trimHistory;
    /**
     * 向聊天发送消息并获取回复
     * @param message 用户消息
     * @returns 助手回复
     */
    sendMessage(message: string): Promise<string>;
    /**
     * 向聊天发送消息并获取流式回复
     * @param message 用户消息
     * @returns 助手回复流
     */
    streamMessage(message: string): Promise<AsyncIterable<string>>;
    /**
     * 收集流式响应并添加到历史
     * @private
     * @param stream 原始响应流
     * @returns 打包的响应流和完整响应Promise
     */
    private collectStreamResponse;
    /**
     * 清空聊天历史
     * @returns 当前聊天构建器实例
     */
    clearHistory(): ChatBuilder;
    /**
     * 获取聊天历史
     * @returns 聊天历史消息数组
     */
    getHistory(): BaseMessage[];
}
