/**
 * 文档加载器模块
 *
 * 用于加载各种格式的文档，并将其分割成适合向量化的小块
 */
import { RecursiveCharacterTextSplitter } from "langchain/text_splitter";
import { Document } from "@langchain/core/documents";
/**
 * 分割文档
 * @param docs 文档数组
 * @param config 分割配置
 * @returns 分割后的文档片段
 */
export async function splitDocuments(docs, config = {}) {
    const { chunkSize = 1000, chunkOverlap = 200 } = config;
    const textSplitter = new RecursiveCharacterTextSplitter({
        chunkSize,
        chunkOverlap,
    });
    return await textSplitter.splitDocuments(docs);
}
/**
 * 从文本字符串创建并分割文档
 * @param text 文本字符串
 * @param metadata 元数据
 * @param config 分割配置
 * @returns 分割后的文档片段
 */
export async function createAndSplitDocument(text, metadata = {}, config = {}) {
    const { chunkSize = 1000, chunkOverlap = 200 } = config;
    const doc = new Document({
        pageContent: text,
        metadata,
    });
    const textSplitter = new RecursiveCharacterTextSplitter({
        chunkSize,
        chunkOverlap,
    });
    return await textSplitter.splitDocuments([doc]);
}
/**
 * 加载和分割文档辅助函数
 * @param loader 文档加载器
 * @param config 分割配置
 * @returns 分割后的文档片段
 */
export async function loadAndSplitDocuments(loader, config = {}) {
    const docs = await loader.load();
    return await splitDocuments(docs, config);
}
