/**
 * Token计数器工具类
 *
 * 用于计算文本或聊天消息中的token数量
 */
/**
 * 通过近似方法估算文本的token数量
 *
 * @param text 需要计算token的文本
 * @returns token数量的估计值
 */
export function estimateTokenCount(text) {
    try {
        // 处理空字符串
        if (!text || text.length === 0) {
            return 0;
        }
        // 假设平均一个token为4个字符
        // 这是一个粗略的估计，实际情况可能因语言和具体内容而异
        const avgCharsPerToken = 4;
        return Math.ceil(text.length / avgCharsPerToken);
    }
    catch (error) {
        console.error("Token计数错误:", error);
        // 返回一个保守的估计
        return Math.ceil((text || "").length / 3);
    }
}
/**
 * 估算消息数组中的token数量
 *
 * @param messages 消息数组
 * @returns token数量的估计值
 */
export function estimateMessagesTokenCount(messages) {
    let totalTokens = 0;
    // 如果没有消息，返回0
    if (!messages || messages.length === 0) {
        return 0;
    }
    // 为每个消息添加基本token开销
    totalTokens += messages.length * 4; // 每个消息有一个固定开销
    // 计算每个消息内容的token数
    for (const message of messages) {
        const content = typeof message.content === "string"
            ? message.content
            : JSON.stringify(message.content);
        totalTokens += estimateTokenCount(content);
    }
    return totalTokens;
}
/**
 * 估算最大可用于上下文的token数量
 *
 * @param modelName 模型名称
 * @returns 估计的最大token上下文长度
 */
export function getModelContextSize(modelName) {
    const modelSizes = {
        "gpt-3.5-turbo": 4096,
        "gpt-3.5-turbo-16k": 16384,
        "gpt-4": 8192,
        "gpt-4-32k": 32768,
        "gpt-4-turbo": 128000,
        "claude-2": 100000,
        "claude-instant-1": 100000,
        "llama-2-7b": 4096,
        "llama-2-13b": 4096,
        "llama-2-70b": 4096,
    };
    return modelSizes[modelName] || 4096; // 默认返回4096
}
