# 模型检测和健康检查

## 概述

在开发基于大语言模型的应用时，了解环境中可用的模型以及它们的健康状况是至关重要的。LLM Node.js 工具包提供了两个主要功能来帮助解决这些问题：

1. **模型检测** - 自动检测环境中可用的大语言模型和嵌入模型
2. **健康检查** - 验证 API 连接和模型可用性，提供详细的诊断信息

这些功能可以帮助你：

- 在应用启动时自动选择最合适的模型
- 在用户配置中提供准确的模型选择列表
- 诊断 API 连接问题并提供具体的解决建议
- 监控模型延迟并在生产环境中检测性能问题

## 模型检测功能

### 基本用法

模型检测功能可以帮助你发现环境中可用的模型：

```typescript
import { ModelProvider, detectAvailableModels } from "llm-nodejs-toolkit";

async function checkAvailableModels() {
  // 设置API密钥
  const apiKeys = {
    [ModelProvider.OPENAI]: process.env.OPENAI_API_KEY,
    [ModelProvider.ANTHROPIC]: process.env.ANTHROPIC_API_KEY,
  };

  // 检测可用的模型
  const availableModels = await detectAvailableModels(
    [ModelProvider.OPENAI, ModelProvider.ANTHROPIC],
    apiKeys
  );

  console.log("可用的OpenAI模型:");
  for (const [modelName, result] of Object.entries(
    availableModels[ModelProvider.OPENAI]
  )) {
    console.log(`${modelName}: ${result.available ? "可用" : "不可用"}`);
    if (result.tokenLimit) {
      console.log(`  Token限制: ${result.tokenLimit}`);
    }
    if (result.details) {
      console.log(`  详情: ${result.details}`);
    }
  }
}

checkAvailableModels();
```

### 返回数据结构

`detectAvailableModels` 函数返回一个嵌套对象，其结构如下：

```typescript
{
  [ModelProvider.OPENAI]: {
    "gpt-3.5-turbo": {
      available: true,
      tokenLimit: 4096
    },
    "gpt-4": {
      available: false,
      details: "API错误: 无权访问"
    },
    // 其他模型...
  },
  [ModelProvider.ANTHROPIC]: {
    // Anthropic模型...
  },
  // 其他提供商...
}
```

每个模型的检测结果包含：

- `available`: 布尔值，表示模型是否可用
- `tokenLimit`: 可选数字，表示模型的最大 token 限制
- `details`: 可选字符串，表示详细信息，特别是在模型不可用时

### 支持的模型提供商

当前支持以下模型提供商：

- `ModelProvider.OPENAI` - OpenAI 模型 (GPT-3.5, GPT-4 等)
- `ModelProvider.AZURE_OPENAI` - Azure OpenAI 服务
- `ModelProvider.ANTHROPIC` - Anthropic 模型 (Claude 等)
- `ModelProvider.LOCAL` - 本地运行的模型

### 检测本地模型

除了 API 模型外，该工具还可以检测运行在本地的模型：

```typescript
import { ModelProvider, detectLocalModels } from "llm-nodejs-toolkit";

async function checkLocalModels() {
  const localModels = await detectLocalModels();

  console.log("本地可用模型:");
  for (const [modelName, result] of Object.entries(localModels)) {
    console.log(`${modelName}: ${result.available ? "可用" : "不可用"}`);
    if (result.details) {
      console.log(`  详情: ${result.details}`);
    }
  }
}

checkLocalModels();
```

## 健康检查功能

### LLM 健康检查

LLM 健康检查功能可以帮助你验证模型连接并诊断问题：

```typescript
import { ModelProvider, checkLLMHealth } from "llm-nodejs-toolkit";

async function testLLMHealth() {
  const healthResult = await checkLLMHealth({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: process.env.OPENAI_API_KEY,
  });

  console.log(`健康检查结果: ${healthResult.success ? "成功" : "失败"}`);

  if (healthResult.latencyMs) {
    console.log(`延迟: ${healthResult.latencyMs}ms`);
  }

  if (!healthResult.success) {
    console.error(`错误: ${healthResult.error}`);
    console.log(`建议: ${healthResult.suggestion}`);
  }

  if (healthResult.info) {
    console.log("额外信息:", healthResult.info);
  }
}

testLLMHealth();
```

### 嵌入模型健康检查

类似地，你可以检查嵌入模型的健康状况：

```typescript
import { ModelProvider, checkEmbeddingHealth } from "llm-nodejs-toolkit";

async function testEmbeddingHealth() {
  const embeddingHealthResult = await checkEmbeddingHealth(
    ModelProvider.OPENAI,
    process.env.OPENAI_API_KEY,
    "text-embedding-ada-002"
  );

  console.log(
    `嵌入健康检查结果: ${embeddingHealthResult.success ? "成功" : "失败"}`
  );

  if (embeddingHealthResult.latencyMs) {
    console.log(`延迟: ${embeddingHealthResult.latencyMs}ms`);
  }

  if (embeddingHealthResult.info?.dimensions) {
    console.log(`嵌入维度: ${embeddingHealthResult.info.dimensions}`);
  }

  if (!embeddingHealthResult.success) {
    console.error(`错误: ${embeddingHealthResult.error}`);
    console.log(`建议: ${embeddingHealthResult.suggestion}`);
  }
}

testEmbeddingHealth();
```

### 健康检查结果结构

健康检查函数返回一个 `HealthCheckResult` 对象，包含以下字段：

```typescript
interface HealthCheckResult {
  /** 是否通过检查 */
  success: boolean;
  /** 延迟（毫秒） */
  latencyMs?: number;
  /** 错误信息 */
  error?: string;
  /** 额外信息 */
  info?: Record<string, any>;
  /** 建议（如有） */
  suggestion?: string;
}
```

### 智能错误建议

健康检查不仅能告诉你是否有问题，还能给出智能建议。例如：

- 如果 API 密钥无效，会建议检查 API 密钥配置
- 如果连接超时，会建议检查网络或代理设置
- 如果触发了速率限制，会建议降低请求频率
- 如果是特定提供商的问题，会提供针对该提供商的具体建议

## 实际应用场景

### 应用启动自检

在应用启动时进行自检，确保所有必要的模型和 API 可用：

```typescript
import {
  ModelProvider,
  checkLLMHealth,
  checkEmbeddingHealth,
} from "llm-nodejs-toolkit";

async function appStartupCheck() {
  console.log("进行应用启动自检...");

  // 检查主要LLM
  const llmHealth = await checkLLMHealth({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: process.env.OPENAI_API_KEY,
  });

  // 检查嵌入模型
  const embeddingHealth = await checkEmbeddingHealth(
    ModelProvider.OPENAI,
    process.env.OPENAI_API_KEY,
    "text-embedding-ada-002"
  );

  // 汇总自检结果
  const allChecksPass = llmHealth.success && embeddingHealth.success;

  if (allChecksPass) {
    console.log("✅ 所有系统正常，应用准备就绪!");
  } else {
    console.error("❌ 启动自检失败:");
    if (!llmHealth.success) {
      console.error(`  - LLM错误: ${llmHealth.error}`);
      console.error(`    建议: ${llmHealth.suggestion}`);
    }
    if (!embeddingHealth.success) {
      console.error(`  - 嵌入模型错误: ${embeddingHealth.error}`);
      console.error(`    建议: ${embeddingHealth.suggestion}`);
    }
    process.exit(1); // 如果自检失败，退出应用
  }
}

appStartupCheck();
```

### 动态模型选择

根据可用模型自动选择最佳模型：

```typescript
import {
  ModelProvider,
  detectAvailableModels,
  RAGBuilder,
} from "llm-nodejs-toolkit";

async function createRAGWithBestModel(documents) {
  // 检测可用模型
  const availableModels = await detectAvailableModels([ModelProvider.OPENAI], {
    [ModelProvider.OPENAI]: process.env.OPENAI_API_KEY,
  });

  // 选择最佳LLM模型
  let bestModel = "gpt-3.5-turbo"; // 默认模型
  const openaiModels = availableModels[ModelProvider.OPENAI];

  // 优先选择GPT-4，如果可用的话
  if (openaiModels["gpt-4"]?.available) {
    bestModel = "gpt-4";
  } else if (openaiModels["gpt-3.5-turbo-16k"]?.available) {
    bestModel = "gpt-3.5-turbo-16k";
  }

  console.log(`选择最佳模型: ${bestModel}`);

  // 创建RAG系统，使用最佳可用模型
  const rag = await new RAGBuilder()
    .addDocuments(documents)
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: bestModel,
      apiKey: process.env.OPENAI_API_KEY,
    })
    .withRAGConfig({
      embeddingModelName: "text-embedding-ada-002",
      vectorStoreType: "memory",
    })
    .build();

  return rag;
}
```

### 服务健康监控

在生产环境中定期监控模型的健康状况：

```typescript
import { ModelProvider, checkLLMHealth } from "llm-nodejs-toolkit";

async function monitorLLMHealth() {
  // 设置监控间隔
  const MONITORING_INTERVAL = 5 * 60 * 1000; // 5分钟

  // 定义监控函数
  const runHealthCheck = async () => {
    try {
      const startTime = Date.now();
      const healthResult = await checkLLMHealth({
        provider: ModelProvider.OPENAI,
        modelName: "gpt-3.5-turbo",
        apiKey: process.env.OPENAI_API_KEY,
      });

      if (healthResult.success) {
        console.log(
          `[${new Date().toISOString()}] 健康检查通过 - 延迟: ${
            healthResult.latencyMs
          }ms`
        );

        // 检查延迟是否异常
        if (healthResult.latencyMs > 2000) {
          console.warn(`⚠️ 延迟异常偏高: ${healthResult.latencyMs}ms`);
        }
      } else {
        console.error(`[${new Date().toISOString()}] ❌ 健康检查失败:`);
        console.error(`  错误: ${healthResult.error}`);
        console.error(`  建议: ${healthResult.suggestion}`);

        // 这里可以触发告警或通知
      }
    } catch (error) {
      console.error(`[${new Date().toISOString()}] 监控过程出错:`, error);
    }

    // 安排下一次检查
    setTimeout(runHealthCheck, MONITORING_INTERVAL);
  };

  // 开始监控
  console.log(
    `[${new Date().toISOString()}] 开始LLM健康监控，间隔: ${
      MONITORING_INTERVAL / 1000
    }秒`
  );
  runHealthCheck();
}

// 在生产环境中启动监控
if (process.env.NODE_ENV === "production") {
  monitorLLMHealth();
}
```

## API 参考

### 模型检测函数

#### `detectAvailableModels(providers, apiKeys)`

检测指定提供商中可用的模型。

- **参数**:

  - `providers`: ModelProvider[] - 要检测的提供商列表
  - `apiKeys`: Record<string, string> - API 密钥映射表

- **返回值**:
  - Promise<Record<ModelProvider, Record<string, ModelDetectionResult>>> - 检测结果

#### `detectOpenAIModels(apiKey)`

专门检测 OpenAI 提供的模型。

- **参数**:

  - `apiKey`: string - OpenAI API 密钥

- **返回值**:
  - Promise<Record<string, ModelDetectionResult>> - OpenAI 模型检测结果

#### `detectLocalModels()`

检测本地可用的模型。

- **返回值**:
  - Promise<Record<string, ModelDetectionResult>> - 本地模型检测结果

### 健康检查函数

#### `checkLLMHealth(config)`

检查 LLM 模型的健康状况。

- **参数**:

  - `config`: LLMConfig - LLM 配置对象
    - `provider`: ModelProvider - 提供商
    - `modelName`: string - 模型名称
    - `apiKey`: string - API 密钥
    - 其他可选配置...

- **返回值**:
  - Promise<HealthCheckResult> - 健康检查结果

#### `checkEmbeddingHealth(provider, apiKey, modelName)`

检查嵌入模型的健康状况。

- **参数**:

  - `provider`: ModelProvider - 提供商
  - `apiKey`: string - API 密钥
  - `modelName`: string - 嵌入模型名称 (默认: "text-embedding-ada-002")

- **返回值**:
  - Promise<HealthCheckResult> - 健康检查结果

## 最佳实践

### 错误处理

- 总是使用 try/catch 包裹模型检测和健康检查函数，确保错误不会中断应用
- 根据健康检查的建议字段提供具体的错误处理逻辑

```typescript
try {
  const healthResult = await checkLLMHealth({
    /*...*/
  });
  if (!healthResult.success) {
    // 根据建议处理问题
    if (healthResult.suggestion.includes("API密钥")) {
      // 处理API密钥问题
    } else if (healthResult.suggestion.includes("网络连接")) {
      // 处理网络问题
    }
  }
} catch (error) {
  console.error("健康检查异常:", error);
}
```

### 性能考虑

- 模型检测和健康检查可能会消耗 API 配额，不要过于频繁地调用
- 考虑对结果进行缓存，避免重复检查
- 在生产环境中，可以使用定时任务，而不是在每个请求时都进行检查

### 备选方案

- 设置模型降级策略，当首选模型不可用时，自动降级到备选模型
- 为不同的模型提供商设置备选方案，增强系统可靠性

```typescript
// 模型降级策略示例
async function getLLMWithFallback() {
  // 尝试首选模型
  const gpt4Health = await checkLLMHealth({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-4",
    apiKey: process.env.OPENAI_API_KEY,
  });

  if (gpt4Health.success) {
    return loadLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-4",
      apiKey: process.env.OPENAI_API_KEY,
    });
  }

  // 降级到备选模型
  console.log("GPT-4不可用，降级到GPT-3.5-Turbo");
  return loadLLM({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: process.env.OPENAI_API_KEY,
  });
}
```

## 故障排除

### 常见问题

1. **API 密钥错误**

   - 确保 API 密钥正确并且未过期
   - 检查是否有足够的 API 额度

2. **网络连接问题**

   - 确保你的网络可以访问 API 端点
   - 如果在中国大陆，可能需要配置代理

3. **速率限制**

   - 如果触发了速率限制，考虑减少请求频率
   - 实现指数退避策略进行重试

4. **本地模型检测失败**
   - 确保模型文件存在且正确加载
   - 检查本地模型的系统要求是否满足

## 下一步

模型检测和健康检查是构建可靠 LLM 应用的基础功能，可以帮助你:

- 自动适应不同的 API 和模型可用性
- 提前发现并解决潜在问题
- 为用户提供准确的模型选项
- 建立完善的应用自检和监控系统

考虑将这些功能集成到你的应用启动流程、配置界面和定期监控中，以提升应用的可靠性和用户体验。
