# RAG (检索增强生成) 模块

## 概述

RAG（检索增强生成）是一种强大的技术，结合了检索系统和生成式 AI 的优势。它通过以下步骤工作：

1. 从文档库中检索与用户问题相关的文本片段
2. 将这些文本片段作为上下文提供给大语言模型
3. 大语言模型基于提供的上下文生成回答

这种方法的优势在于：

- 使 LLM 能够访问其训练数据之外的知识
- 减少幻觉（生成虚假信息）的可能性
- 允许模型引用特定文档中的信息

## 模块组件

LLM Node.js 工具包的 RAG 模块由以下主要组件组成：

### 1. 文档加载器

负责从各种来源（文本、PDF、网页等）加载文档，并将其转换为适合处理的格式。

### 2. 文档分割器

将文档分割成适当大小的块，使其适合进行嵌入和检索。

### 3. 嵌入模型

将文本转换为向量表示形式，使其可以进行相似性搜索。

### 4. 向量存储

存储文档块的向量表示及其原始内容，支持基于相似性的快速检索。

### 5. 检索器

基于用户查询检索相关文档块的组件。

### 6. RAG 链

将检索到的文档块与用户查询一起提供给 LLM，并生成最终回答的处理流程。

## 使用方法

### 基本用法

最简单的 RAG 应用可以通过`RAGBuilder`类快速构建：

```typescript
import {
  RAGBuilder,
  ModelProvider,
  createAndSplitDocument,
} from "llm-nodejs-toolkit";

async function main() {
  // 准备文档
  const docs = await createAndSplitDocument(
    "这是一份示例文档，包含了一些信息。Node.js是一个JavaScript运行时环境。",
    { source: "示例" }
  );

  // 创建RAG系统
  const rag = await new RAGBuilder()
    .addDocuments(docs)
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: process.env.OPENAI_API_KEY,
    })
    .withRAGConfig({
      embeddingModelName: "text-embedding-ada-002",
      vectorStoreType: "memory",
    })
    .build();

  // 查询RAG系统
  const answer = await rag.query("什么是Node.js？");
  console.log(answer);
}

main().catch(console.error);
```

### 高级配置

RAG 模块支持多种高级配置选项：

```typescript
const rag = await new RAGBuilder()
  .addDocuments(docs)
  .withLLM({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: process.env.OPENAI_API_KEY,
  })
  .withRAGConfig({
    embeddingModelName: "text-embedding-ada-002", // 嵌入模型名称
    vectorStoreType: "memory", // 向量存储类型
    topK: 3, // 检索的文档块数量
    similarityThreshold: 0.7, // 相似度阈值
    includeMetadata: true, // 是否包含元数据
    systemPrompt: "你是一个专业的助手，根据提供的上下文回答问题。", // 自定义系统提示
  })
  .build();
```

### 添加文档的方式

你可以通过多种方式向 RAG 系统添加文档：

1. **直接添加文本**:

```typescript
const docs = await createAndSplitDocument("这是文档内容...", {
  source: "文档来源",
});
ragBuilder.addDocuments(docs);
```

2. **加载本地文件**:

```typescript
import { loadDocumentsFromFiles } from "llm-nodejs-toolkit";

const docs = await loadDocumentsFromFiles([
  "./data/file1.txt",
  "./data/file2.pdf",
]);
ragBuilder.addDocuments(docs);
```

3. **加载网页内容**:

```typescript
import { loadDocumentsFromWeb } from "llm-nodejs-toolkit";

const docs = await loadDocumentsFromWeb([
  "https://example.com/page1",
  "https://example.com/page2",
]);
ragBuilder.addDocuments(docs);
```

## API 参考

### RAGBuilder 类

`RAGBuilder`是构建 RAG 系统的主要类，提供以下方法：

#### `addDocuments(documents: Document[]): RAGBuilder`

添加文档到 RAG 系统。

- **参数**:

  - `documents`: Document[] - 要添加的文档数组

- **返回值**: RAGBuilder 实例（用于链式调用）

#### `withLLM(config: LLMConfig): RAGBuilder`

设置要使用的大语言模型。

- **参数**:

  - `config`: LLMConfig - LLM 配置对象
    - `provider`: ModelProvider - 模型提供商（如 OPENAI）
    - `modelName`: string - 模型名称
    - `apiKey`: string - API 密钥
    - `temperature`: number - 温度参数（可选）
    - `maxTokens`: number - 最大生成 token 数（可选）

- **返回值**: RAGBuilder 实例（用于链式调用）

#### `withRAGConfig(config: RAGConfig): RAGBuilder`

设置 RAG 系统的配置。

- **参数**:

  - `config`: RAGConfig - RAG 配置对象
    - `embeddingModelName`: string - 嵌入模型名称
    - `vectorStoreType`: string - 向量存储类型
    - `topK`: number - 检索的文档块数量（可选）
    - `similarityThreshold`: number - 相似度阈值（可选）
    - `includeMetadata`: boolean - 是否包含元数据（可选）
    - `systemPrompt`: string - 自定义系统提示（可选）

- **返回值**: RAGBuilder 实例（用于链式调用）

#### `build(): Promise<RAG>`

构建 RAG 系统。

- **返回值**: Promise<RAG> - RAG 系统实例

### RAG 类

`RAG`是 RAG 系统的实例，提供以下方法：

#### `query(question: string): Promise<string>`

向 RAG 系统提问，获取基于文档的回答。

- **参数**:

  - `question`: string - 用户问题

- **返回值**: Promise<string> - 基于文档的回答

#### `addDocuments(documents: Document[]): Promise<void>`

向已构建的 RAG 系统添加更多文档。

- **参数**:

  - `documents`: Document[] - 要添加的文档数组

- **返回值**: Promise<void>

#### `getRelevantDocuments(question: string): Promise<Document[]>`

获取与问题相关的文档块，但不生成回答。

- **参数**:

  - `question`: string - 用户问题

- **返回值**: Promise<Document[]> - 相关文档块数组

## 相关工具函数

### `createAndSplitDocument(text: string, metadata?: Record<string, any>): Promise<Document[]>`

创建并分割文档。

- **参数**:

  - `text`: string - 文档文本
  - `metadata`: Record<string, any> - 文档元数据（可选）

- **返回值**: Promise<Document[]> - 分割后的文档块数组

### `loadDocumentsFromFiles(filePaths: string[]): Promise<Document[]>`

从本地文件加载文档。

- **参数**:

  - `filePaths`: string[] - 文件路径数组

- **返回值**: Promise<Document[]> - 加载的文档数组

### `loadDocumentsFromWeb(urls: string[]): Promise<Document[]>`

从网页加载文档。

- **参数**:

  - `urls`: string[] - URL 数组

- **返回值**: Promise<Document[]> - 加载的文档数组

## 实际应用场景

### 客服知识库

创建一个客服机器人，能够回答关于产品的问题：

```typescript
// 加载产品文档
const productDocs = await loadDocumentsFromFiles([
  "./docs/product-specs.pdf",
  "./docs/user-guide.pdf",
  "./docs/faq.txt",
]);

// 创建RAG系统
const customerSupportRAG = await new RAGBuilder()
  .addDocuments(productDocs)
  .withLLM({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: process.env.OPENAI_API_KEY,
  })
  .withRAGConfig({
    embeddingModelName: "text-embedding-ada-002",
    vectorStoreType: "memory",
    systemPrompt: "你是一个专业的客服助手，根据产品文档回答客户问题。",
  })
  .build();

// 回答客户问题
const answer = await customerSupportRAG.query("如何重置我的设备密码？");
console.log(answer);
```

### 个人文档助手

创建一个可以查询个人笔记和文档的助手：

```typescript
// 加载个人笔记和文档
const personalDocs = await loadDocumentsFromFiles([
  "./notes/meeting-notes.txt",
  "./notes/ideas.md",
  "./documents/project-plan.docx",
]);

// 创建RAG系统
const personalAssistant = await new RAGBuilder()
  .addDocuments(personalDocs)
  .withLLM({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-4",
    apiKey: process.env.OPENAI_API_KEY,
  })
  .withRAGConfig({
    embeddingModelName: "text-embedding-ada-002",
    vectorStoreType: "memory",
    topK: 5,
    systemPrompt: "你是我的个人助手，帮助我查找和总结我的笔记和文档中的信息。",
  })
  .build();

// 查询个人文档
const summary = await personalAssistant.query(
  "总结一下我们上周会议的主要决定。"
);
console.log(summary);
```

## 最佳实践

### 文档分割

- **块大小控制**: 文档块不要太大也不要太小。太大的块会导致检索不精确，太小的块会丢失上下文信息。
- **重叠设置**: 在分割文档时设置一定的重叠，可以帮助保持上下文连贯性。

### 提示工程

- **明确指示**: 在系统提示中明确告诉模型如何使用检索到的上下文。
- **引用要求**: 要求模型在回答中引用文档中的具体信息。

### 性能优化

- **过滤无关文档**: 在添加文档前进行预处理，过滤掉无关内容。
- **索引更新策略**: 定期更新文档索引，保持信息的最新性。

## 常见问题

### 问题: 为什么我的 RAG 系统返回的答案不准确？

可能的原因：

- 检索到的文档块不相关
- 检索的文档块数量不足
- 文档分割不当，导致上下文丢失
- 向量嵌入模型不适合你的文档类型

解决方案：

- 调整`topK`参数，增加检索的文档块数量
- 调整文档分割配置，减小块大小并增加重叠
- 尝试不同的嵌入模型
- 对系统提示进行优化

### 问题: 我的查询速度很慢，如何优化？

解决方案：

- 使用持久化的向量存储而不是内存存储
- 对大型文档集合使用索引
- 减少检索的文档块数量（降低`topK`值）
- 使用更轻量级的嵌入模型

### 问题: 如何处理非文本数据（如图片、视频）？

解决方案：

- 对于图片，可以使用图像识别 API 提取描述，然后将描述作为文本添加到 RAG 系统
- 对于视频，可以使用语音转文字服务提取文本内容
- 对于表格数据，可以将其转换为结构化文本表示

## 下一步

为了进一步提升你的 RAG 系统，考虑：

- 探索不同的向量存储选项，如 Pinecone、Milvus 或 Weaviate
- 实现多模态 RAG，结合文本和图像处理
- 添加用户反馈机制，不断优化回答质量
- 与 Agent 模块集成，创建能够访问文档的智能代理
