# 安装指南

本指南将帮助你正确设置和安装 LLM Node.js 工具包。

## 环境要求

在安装前，请确保你的环境满足以下要求：

- **Node.js**: v18.0.0 或更高版本 (推荐 v20+)
- **包管理器**: npm、yarn 或 pnpm (推荐 pnpm)

你可以通过以下命令检查你的 Node.js 版本：

```bash
node --version
```

## 安装步骤

### 1. 使用 pnpm 安装（推荐）

```bash
# 创建新项目
mkdir my-llm-app
cd my-llm-app
pnpm init

# 安装 llm-nodejs-toolkit
pnpm add llm-nodejs-toolkit
```

### 2. 使用 npm 安装

```bash
# 创建新项目
mkdir my-llm-app
cd my-llm-app
npm init -y

# 安装 llm-nodejs-toolkit
npm install llm-nodejs-toolkit
```

### 3. 使用 yarn 安装

```bash
# 创建新项目
mkdir my-llm-app
cd my-llm-app
yarn init -y

# 安装 llm-nodejs-toolkit
yarn add llm-nodejs-toolkit
```

## 配置 TypeScript（可选但推荐）

为了获得更好的开发体验，我们强烈推荐使用 TypeScript：

```bash
# 使用 pnpm
pnpm add -D typescript @types/node

# 使用 npm
npm install --save-dev typescript @types/node

# 使用 yarn
yarn add --dev typescript @types/node
```

然后创建一个基本的 `tsconfig.json` 文件：

```json
{
  "compilerOptions": {
    "target": "ES2022",
    "module": "NodeNext",
    "moduleResolution": "NodeNext",
    "esModuleInterop": true,
    "strict": true,
    "outDir": "dist",
    "declaration": true,
    "sourceMap": true
  },
  "include": ["src/**/*"],
  "exclude": ["node_modules", "**/*.test.ts"]
}
```

## 设置 API 密钥

LLM Node.js 工具包支持多种大语言模型提供商，你需要为你计划使用的提供商设置 API 密钥。

### 方法 1: 使用环境变量（推荐）

创建一个 `.env` 文件，并添加你的 API 密钥：

```bash
# .env 文件
OPENAI_API_KEY=你的OpenAI密钥
ANTHROPIC_API_KEY=你的Anthropic密钥
AZURE_OPENAI_API_KEY=你的Azure OpenAI密钥
AZURE_OPENAI_ENDPOINT=你的Azure OpenAI端点
```

然后，安装并配置 dotenv 包：

```bash
# 使用 pnpm
pnpm add dotenv

# 使用 npm
npm install dotenv

# 使用 yarn
yarn add dotenv
```

在你的代码中加载环境变量：

```typescript
import "dotenv/config";
```

### 方法 2: 直接在代码中设置

你也可以在代码中直接设置 API 密钥，但请注意不要将密钥提交到代码仓库中：

```typescript
import { RAGBuilder, ModelProvider } from "llm-nodejs-toolkit";

const rag = new RAGBuilder()
  .withLLM({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: "你的OpenAI密钥", // 在此处设置API密钥
  })
  .build();
```

## 验证安装

安装完成后，你可以创建一个简单的脚本来验证安装是否成功：

创建一个 `test.ts` 文件（或 `test.js` 如果你不使用 TypeScript）：

```typescript
// test.ts
import { checkLLMHealth, ModelProvider } from "llm-nodejs-toolkit";

async function testInstallation() {
  try {
    // 使用环境变量中的API密钥进行健康检查
    const healthResult = await checkLLMHealth({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      // 如果你使用了.env文件，这里不需要显式提供apiKey
    });

    console.log("健康检查结果:", healthResult);

    if (healthResult.success) {
      console.log("🎉 恭喜！安装成功，并且可以连接到OpenAI API！");
      console.log(`延迟: ${healthResult.latencyMs}ms`);
    } else {
      console.error("❌ 连接失败:", healthResult.error);
      console.log("建议:", healthResult.suggestion);
    }
  } catch (error) {
    console.error("❌ 测试过程中出错:", error);
  }
}

testInstallation();
```

运行测试脚本：

```bash
# 如果使用 TypeScript
npx tsx test.ts

# 如果使用 JavaScript
node test.js
```

## 故障排除

如果你在安装或配置过程中遇到问题，以下是一些常见问题的解决方法：

### API 连接问题

- 确保你的 API 密钥是正确的
- 检查你的网络连接，确保可以访问 API 端点
- 如果你在中国大陆，可能需要设置代理才能访问某些 API

### 依赖冲突

如果遇到依赖冲突，可以尝试以下解决方案：

```bash
# 使用 pnpm
pnpm install --force

# 使用 npm
npm install --legacy-peer-deps

# 使用 yarn
yarn install --force
```

### Node.js 版本问题

如果你使用的 Node.js 版本过低，可能会导致兼容性问题。建议使用 nvm 或 volta 等工具管理 Node.js 版本：

```bash
# 使用 nvm 安装最新的 LTS 版本
nvm install --lts
nvm use --lts
```

## 下一步

现在你已经成功安装了 LLM Node.js 工具包，可以继续阅读[快速入门](./quickstart.md)指南，开始构建你的第一个 LLM 应用！
