# 快速入门

本指南将帮助你在 15 分钟内使用 LLM Node.js 工具包构建一个简单的应用。我们将创建三个小项目：一个简单的聊天机器人、一个基于文档的问答系统，以及一个能够执行任务的智能代理。

## 准备工作

在开始之前，请确保你已经：

1. 安装了 Node.js (v18+)
2. 安装了 LLM Node.js 工具包（参见[安装指南](./installation.md)）
3. 设置了 OpenAI API 密钥

创建一个新项目：

```bash
mkdir llm-quickstart
cd llm-quickstart
pnpm init -y
pnpm add llm-nodejs-toolkit dotenv
pnpm add -D typescript @types/node tsx
```

创建一个 `.env` 文件并添加你的 API 密钥：

```
OPENAI_API_KEY=你的密钥
```

创建 `tsconfig.json` 文件：

```json
{
  "compilerOptions": {
    "target": "ES2022",
    "module": "NodeNext",
    "moduleResolution": "NodeNext",
    "esModuleInterop": true,
    "strict": true,
    "outDir": "dist"
  },
  "include": ["src/**/*"]
}
```

## 示例 1：简单聊天机器人

创建 `src/chat.ts` 文件：

```typescript
// src/chat.ts
import "dotenv/config";
import { ChatBuilder, ModelProvider } from "llm-nodejs-toolkit";
import readline from "readline";

async function main() {
  // 创建聊天应用
  const chat = new ChatBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      // apiKey会自动从环境变量中获取
    })
    .withSystemPrompt("你是一个友好的AI助手，擅长用简洁和友好的方式回答问题。")
    .withConfig({
      memoryMessageLimit: 10, // 保留最近10条消息的上下文
    })
    .build();

  // 创建命令行交互界面
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
  });

  console.log("欢迎使用聊天机器人！输入 'exit' 退出。");

  // 开始聊天循环
  const askQuestion = () => {
    rl.question("你: ", async (input) => {
      if (input.toLowerCase() === "exit") {
        console.log("再见！");
        rl.close();
        return;
      }

      try {
        // 发送消息并获取回复
        const response = await chat.sendMessage(input);
        console.log(`AI助手: ${response}`);
      } catch (error) {
        console.error("发生错误:", error);
      }

      // 继续对话
      askQuestion();
    });
  };

  // 启动对话
  askQuestion();
}

main().catch(console.error);
```

运行你的聊天机器人：

```bash
npx tsx src/chat.ts
```

## 示例 2：文档问答系统 (RAG)

创建 `src/rag.ts` 文件：

```typescript
// src/rag.ts
import "dotenv/config";
import {
  RAGBuilder,
  createAndSplitDocument,
  ModelProvider,
} from "llm-nodejs-toolkit";
import readline from "readline";

async function main() {
  console.log("正在准备文档问答系统...");

  // 准备示例文档
  const documentText = `
  Node.js是一个基于Chrome V8引擎的JavaScript运行环境。它使JavaScript可以在浏览器之外运行，
  让JavaScript成为一种服务器端脚本语言。Node.js采用事件驱动、非阻塞I/O模型，使其轻量且高效，
  非常适合构建数据密集型实时应用程序。

  Node.js主要特点：
  1. 异步事件驱动架构
  2. 单线程设计
  3. 非阻塞I/O
  4. 模块化系统

  Node.js安装非常简单，可以从官网下载安装包，也可以使用nvm（Node Version Manager）进行安装和管理。
  对于初学者，推荐学习基础JavaScript语法，理解回调函数、Promise和async/await等异步编程概念。
  `;

  // 分割文档
  const docs = await createAndSplitDocument(documentText, {
    source: "Node.js简介",
  });

  // 创建RAG系统
  const rag = await new RAGBuilder()
    .addDocuments(docs)
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      // apiKey会自动从环境变量中获取
    })
    .withRAGConfig({
      embeddingModelName: "text-embedding-ada-002", // 嵌入模型
      vectorStoreType: "memory", // 使用内存向量存储
    })
    .build();

  console.log("文档问答系统已准备就绪！输入 'exit' 退出。");
  console.log("请输入关于Node.js的问题:");

  // 创建命令行交互界面
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
  });

  // 开始问答循环
  const askQuestion = () => {
    rl.question("问题: ", async (question) => {
      if (question.toLowerCase() === "exit") {
        console.log("再见！");
        rl.close();
        return;
      }

      try {
        console.log("正在思考...");
        // 查询RAG系统
        const answer = await rag.query(question);
        console.log(`回答: ${answer}`);
      } catch (error) {
        console.error("发生错误:", error);
      }

      // 继续问答
      askQuestion();
    });
  };

  // 启动问答
  askQuestion();
}

main().catch(console.error);
```

运行文档问答系统：

```bash
npx tsx src/rag.ts
```

## 示例 3：智能代理 (Agent)

创建 `src/agent.ts` 文件：

```typescript
// src/agent.ts
import "dotenv/config";
import {
  AgentBuilder,
  ModelProvider,
  createCalculatorTool,
  createDateTimeTool,
} from "llm-nodejs-toolkit";
import readline from "readline";

async function main() {
  console.log("正在初始化智能代理...");

  // 创建Agent
  const agent = await new AgentBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      // apiKey会自动从环境变量中获取
    })
    .withTool(createCalculatorTool()) // 添加计算器工具
    .withTool(createDateTimeTool()) // 添加日期时间工具
    .build();

  console.log("智能代理已准备就绪！输入 'exit' 退出。");
  console.log(
    "请输入你的问题或任务（例如：'今天是星期几？'或'计算253乘以14'）:"
  );

  // 创建命令行交互界面
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
  });

  // 开始交互循环
  const askQuestion = () => {
    rl.question("任务: ", async (input) => {
      if (input.toLowerCase() === "exit") {
        console.log("再见！");
        rl.close();
        return;
      }

      try {
        console.log("正在处理...");
        // 执行代理
        const result = await agent.execute(input);
        console.log(`结果: ${result.output}`);
      } catch (error) {
        console.error("发生错误:", error);
      }

      // 继续交互
      askQuestion();
    });
  };

  // 启动交互
  askQuestion();
}

main().catch(console.error);
```

运行智能代理：

```bash
npx tsx src/agent.ts
```

## 组合应用

现在，让我们创建一个小型菜单程序，集成上述三个功能：

创建 `src/index.ts` 文件：

```typescript
// src/index.ts
import "dotenv/config";
import readline from "readline";
import { exec } from "child_process";
import { promisify } from "util";

const execPromise = promisify(exec);

async function main() {
  console.log("=== LLM Node.js 工具包快速入门 ===");
  console.log("1. 聊天机器人");
  console.log("2. 文档问答系统 (RAG)");
  console.log("3. 智能代理 (Agent)");
  console.log("0. 退出");

  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
  });

  const askChoice = () => {
    rl.question("请选择 (0-3): ", async (choice) => {
      switch (choice) {
        case "1":
          // 运行聊天机器人
          await runExample("chat");
          break;
        case "2":
          // 运行文档问答系统
          await runExample("rag");
          break;
        case "3":
          // 运行智能代理
          await runExample("agent");
          break;
        case "0":
          console.log("再见！");
          rl.close();
          return;
        default:
          console.log("无效选择，请重试");
      }
      askChoice();
    });
  };

  askChoice();
}

async function runExample(type: "chat" | "rag" | "agent") {
  try {
    console.log(`启动${type}应用...`);
    await execPromise(`npx tsx src/${type}.ts`);
  } catch (error) {
    console.error(`运行${type}应用时出错:`, error);
  }
}

main().catch(console.error);
```

运行主程序：

```bash
npx tsx src/index.ts
```

## 探索更多

恭喜！你已经成功构建了三个不同类型的 LLM 应用。要进一步探索工具包的功能，可以查看：

- [RAG 模块详解](./core-features/rag.md) - 了解更多 RAG 功能
- [Agent 模块详解](./core-features/agent.md) - 学习如何创建自定义工具
- [Chat 模块详解](./core-features/chat.md) - 深入了解聊天功能
- [高级功能](./advanced-features/README.md) - 探索更高级的功能

通过这个快速入门，你已经掌握了使用 LLM Node.js 工具包的基础知识，可以开始构建你自己的应用了！
