/**
 * Agent示例
 *
 * 本示例展示如何使用AgentBuilder和各种工具创建一个智能Agent
 * 使用方法：
 * 1. 将API_KEY替换为你的OpenAI API密钥
 * 2. 运行 npx tsx agent-example.ts
 */

import {
  ModelProvider,
  AgentBuilder,
  createCalculatorTool,
  createDateTimeTool,
  createSearchTool,
  createToolFromInterface,
} from "../src/index.js";

// 替换为你的API密钥
const API_KEY = "your-api-key";

// 创建一个自定义天气工具（模拟）
const weatherTool = createToolFromInterface({
  name: "weather",
  description: "查询特定城市的天气信息",
  schema: {
    city: { type: "string", description: "城市名称" },
  },
  func: async ({ city }) => {
    // 注意：这只是模拟数据，实际应用中应该调用真实的天气API
    const cities = {
      北京: { temp: "23°C", condition: "晴朗", humidity: "42%" },
      上海: { temp: "26°C", condition: "多云", humidity: "68%" },
      广州: { temp: "30°C", condition: "小雨", humidity: "75%" },
      深圳: { temp: "29°C", condition: "阴天", humidity: "70%" },
    };

    if (cities[city]) {
      return `${city}当前天气：${cities[city].temp}，${cities[city].condition}，湿度${cities[city].humidity}`;
    } else {
      return `抱歉，没有找到${city}的天气信息`;
    }
  },
});

async function main() {
  console.log("正在初始化Agent...");

  // 创建Agent
  const agent = await new AgentBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: API_KEY,
      temperature: 0.5,
    })
    // 添加工具
    .withTool(createCalculatorTool())
    .withTool(createDateTimeTool())
    .withTool(createSearchTool())
    .withTool(weatherTool)
    .build();

  console.log("Agent初始化完成！");
  console.log("-------------------");

  // 示例查询
  const questions = [
    "今天是几号？",
    "计算一下847乘以23等于多少？",
    "北京现在的天气怎么样？",
    "请帮我查询关于量子计算的信息",
    "先告诉我现在的日期，然后计算256除以8的结果",
  ];

  for (const question of questions) {
    console.log(`用户: ${question}`);

    // 执行查询
    const result = await agent.execute(question);
    console.log(`Agent: ${result.output}`);

    // 如果有中间步骤（工具调用），显示它们
    if (result.intermediateSteps.length > 0) {
      console.log("\n工具调用步骤:");
      for (const step of result.intermediateSteps) {
        if (step.action) {
          console.log(`- 调用工具: ${step.action.tool}`);
          console.log(`  输入: ${JSON.stringify(step.action.toolInput)}`);
          console.log(`  输出: ${step.observation}`);
        }
      }
    }

    console.log("-------------------");
  }
}

main().catch((error) => {
  console.error("出错了:", error);
});
