/**
 * LLM Node.js Toolkit 示例入口
 *
 * 这个文件展示了如何导入和使用库的主要功能
 * 本文件仅用于展示API，不会实际运行
 */

// 导入主要组件
import {
  // 基础类型和枚举
  ModelProvider,
  AIMessage,
  HumanMessage,
  SystemMessage,

  // RAG相关
  RAGBuilder,
  createAndSplitDocument,

  // 聊天相关
  ChatBuilder,

  // Agent相关
  AgentBuilder,
  createCalculatorTool,
  createDateTimeTool,
  createSearchTool,
  createToolFromInterface,

  // 工具函数
  estimateTokenCount,
  createEmbeddings,
  loadLLM,
} from "../src/index.js";

/**
 * 这里简单展示了几个主要模块的使用方式
 * 要查看完整的运行示例，请查看：
 * - chat-example.ts：聊天功能示例
 * - rag-example.ts：检索增强生成示例
 * - agent-example.ts：智能代理示例
 */

// 示例API密钥（使用前请替换）
const API_KEY = "your-api-key";

// 简易聊天示例（查看完整示例请运行chat-example.ts）
async function chatExample() {
  const chat = new ChatBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: API_KEY,
    })
    .withSystemPrompt("你是一个有帮助的AI助手。")
    .build();

  const response = await chat.sendMessage("你好！");
  console.log(response);
}

// 简易RAG示例（查看完整示例请运行rag-example.ts）
async function ragExample() {
  const docs = await createAndSplitDocument(
    "人工智能是计算机科学的一个分支，致力于创造能够模拟人类智能的机器。",
    { source: "人工智能简介" }
  );

  const rag = await new RAGBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: API_KEY,
    })
    .withEmbedding({
      provider: ModelProvider.OPENAI,
      modelName: "text-embedding-ada-002",
      apiKey: API_KEY,
    })
    .addDocuments(docs)
    .build();

  const answer = await rag.query("什么是人工智能？");
  console.log(answer);
}

// 简易Agent示例（查看完整示例请运行agent-example.ts）
async function agentExample() {
  const agent = await new AgentBuilder()
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: API_KEY,
    })
    .withTool(createCalculatorTool())
    .withTool(createDateTimeTool())
    .build();

  const result = await agent.execute("计算23乘以45是多少？");
  console.log(result.output);
}

// 注意：这个文件仅用于展示API，不会实际运行上述函数
console.log("这是一个API展示文件，请运行具体的示例文件来测试功能：");
console.log("- npx tsx chat-example.ts");
console.log("- npx tsx rag-example.ts");
console.log("- npx tsx agent-example.ts");
