/**
 * 模型检测和健康检查示例
 *
 * 本示例展示如何使用模型检测和健康检查功能
 * 使用方法：
 * 1. 将API_KEY替换为你的OpenAI API密钥
 * 2. 运行 npx tsx model-health-check-example.ts
 */

import {
  ModelProvider,
  detectAvailableModels,
  checkLLMHealth,
  checkEmbeddingHealth,
} from "../src/index.js";

// 替换为你的API密钥
const API_KEY = "your-api-key";

async function main() {
  console.log("=== 模型检测和健康检查示例 ===");
  console.log("正在检测可用的模型...");

  // 检测可用的模型
  const modelResults = await detectAvailableModels([ModelProvider.OPENAI], {
    [ModelProvider.OPENAI]: API_KEY,
  });

  // 打印OpenAI模型检测结果
  console.log("\n=== OpenAI模型检测结果 ===");
  for (const [modelName, result] of Object.entries(
    modelResults[ModelProvider.OPENAI]
  )) {
    console.log(`${modelName}: ${result.available ? "可用" : "不可用"}`);
    if (result.details) {
      console.log(`  详情: ${result.details}`);
    }
    if (result.tokenLimit) {
      console.log(`  Token限制: ${result.tokenLimit}`);
    }
  }

  // 执行LLM健康检查
  console.log("\n=== 执行LLM健康检查 ===");
  const llmHealthResult = await checkLLMHealth({
    provider: ModelProvider.OPENAI,
    modelName: "gpt-3.5-turbo",
    apiKey: API_KEY,
  });

  console.log(`健康检查结果: ${llmHealthResult.success ? "成功" : "失败"}`);
  console.log(`延迟: ${llmHealthResult.latencyMs}ms`);

  if (llmHealthResult.error) {
    console.log(`错误: ${llmHealthResult.error}`);
  }

  if (llmHealthResult.suggestion) {
    console.log(`建议: ${llmHealthResult.suggestion}`);
  }

  // 执行嵌入模型健康检查
  console.log("\n=== 执行嵌入模型健康检查 ===");
  const embeddingHealthResult = await checkEmbeddingHealth(
    ModelProvider.OPENAI,
    API_KEY,
    "text-embedding-ada-002"
  );

  console.log(
    `健康检查结果: ${embeddingHealthResult.success ? "成功" : "失败"}`
  );
  console.log(`延迟: ${embeddingHealthResult.latencyMs}ms`);

  if (embeddingHealthResult.error) {
    console.log(`错误: ${embeddingHealthResult.error}`);
  }

  if (embeddingHealthResult.info?.dimensions) {
    console.log(`嵌入维度: ${embeddingHealthResult.info.dimensions}`);
  }

  if (embeddingHealthResult.suggestion) {
    console.log(`建议: ${embeddingHealthResult.suggestion}`);
  }
}

main().catch((error) => {
  console.error("出错了:", error);
});
