/**
 * 提示模板构建器示例
 *
 * 本示例展示如何使用PromptBuilder和预定义模板函数
 * 使用方法：
 * 1. 将API_KEY替换为你的OpenAI API密钥
 * 2. 运行 npx tsx prompt-builder-example.ts
 */

import {
  ModelProvider,
  PromptBuilder,
  createRAGPromptTemplate,
  createChatPromptTemplate,
  createTranslationPromptTemplate,
  createExtractionPromptTemplate,
} from "../src/index.js";
import { ChatOpenAI } from "@langchain/openai";
import { StringOutputParser } from "@langchain/core/output_parsers";

// 替换为你的API密钥
const API_KEY = "your-api-key";

async function main() {
  console.log("=== 提示模板构建器示例 ===");

  // 创建LLM实例
  const llm = new ChatOpenAI({
    openAIApiKey: API_KEY,
    modelName: "gpt-3.5-turbo",
    temperature: 0.7,
  });

  // 输出解析器
  const outputParser = new StringOutputParser();

  // 1. 使用PromptBuilder构建自定义提示模板
  console.log("\n=== 自定义提示模板示例 ===");
  const customPrompt = new PromptBuilder()
    .addSystemMessage(
      "你是一个热情的美食专家，擅长解释各种菜品的做法和背景知识。"
    )
    .addExampleDialog(
      "请介绍一下意大利面的历史",
      "意大利面的历史可以追溯到公元前4世纪的意大利。虽然很多人认为意大利面是马可·波罗从中国带回来的，但考古证据表明意大利人在马可·波罗之前就已经食用类似的食品。意大利各地区发展出了数百种不同形状和大小的面食，每种都有其独特的用途和搭配的酱料。"
    )
    .addHumanMessage("请问{food}有什么特色？")
    .build();

  const foodChain = customPrompt.pipe(llm).pipe(outputParser);
  const foodResult = await foodChain.invoke({ food: "四川火锅" });
  console.log("自定义美食提示结果:");
  console.log(foodResult);

  // 2. 使用预定义的RAG提示模板
  console.log("\n=== RAG提示模板示例 ===");
  const ragPrompt = createRAGPromptTemplate(
    "你是一个技术文档助手，擅长从提供的文档中回答问题。"
  );

  const ragChain = ragPrompt.pipe(llm).pipe(outputParser);
  const ragResult = await ragChain.invoke({
    context:
      "Node.js是一个基于Chrome V8引擎的JavaScript运行环境，使JavaScript可以在浏览器之外运行。Node.js使用事件驱动、非阻塞I/O模型，使其轻量且高效。",
    question: "Node.js的主要特点是什么？",
  });

  console.log("RAG提示结果:");
  console.log(ragResult);

  // 3. 使用预定义的翻译提示模板
  console.log("\n=== 翻译提示模板示例 ===");
  const translationPrompt = createTranslationPromptTemplate("英语");

  const translationChain = translationPrompt.pipe(llm).pipe(outputParser);
  const translationResult = await translationChain.invoke({
    targetLanguage: "英语",
    text: "人工智能是计算机科学的一个分支，致力于创造能够模拟人类智能的机器。",
  });

  console.log("翻译提示结果:");
  console.log(translationResult);

  // 4. 使用预定义的内容提取提示模板
  console.log("\n=== 内容提取提示模板示例 ===");
  const extractionPrompt = createExtractionPromptTemplate();

  const extractionChain = extractionPrompt.pipe(llm).pipe(outputParser);
  const extractionResult = await extractionChain.invoke({
    text: "张三今年28岁，来自北京，是一名软件工程师，有5年工作经验，精通JavaScript和Python。",
    schema: `{
      "name": "姓名",
      "age": "年龄(数字)",
      "location": "所在地",
      "occupation": "职业",
      "experience": "工作经验(年数)",
      "skills": ["技能列表"]
    }`,
  });

  console.log("内容提取结果:");
  console.log(extractionResult);
}

main().catch((error) => {
  console.error("出错了:", error);
});
