/**
 * RAG示例
 *
 * 本示例展示如何使用RAGBuilder创建一个检索增强生成(RAG)应用
 * 使用方法：
 * 1. 将API_KEY替换为你的OpenAI API密钥
 * 2. 运行 npx tsx rag-example.ts
 */

import {
  ModelProvider,
  RAGBuilder,
  createAndSplitDocument,
} from "../src/index.js";

// 替换为你的API密钥
const API_KEY = "your-api-key";

// 示例文档
const DOCUMENTS = [
  {
    text: `Node.js是一个基于Chrome V8引擎的JavaScript运行环境，使JavaScript可以在浏览器之外运行。
Node.js使用事件驱动、非阻塞I/O模型，使其轻量且高效，非常适合构建运行在分布式设备上的数据密集型实时应用程序。
Node.js的包管理器npm是全球最大的开源库生态系统，可以轻松共享和重用代码。`,
    metadata: { source: "技术文档", topic: "Node.js", id: "1" },
  },
  {
    text: `TypeScript是JavaScript的超集，它添加了静态类型系统，由Microsoft开发和维护。
TypeScript通过在代码运行前捕获错误，增强了大型应用程序的开发和维护能力。
使用TypeScript可以提高代码质量、增强开发者体验，并改善团队协作。`,
    metadata: { source: "技术文档", topic: "TypeScript", id: "2" },
  },
  {
    text: `大语言模型(LLM)是一种基于深度学习的先进人工智能系统，通过大规模文本数据训练，能够理解和生成类人的自然语言。
这些模型具有上下文理解、知识推理和多样化内容生成的能力，被广泛应用于对话系统、内容创作和知识检索等领域。
虽然LLM表现出惊人的语言能力，但它们仍面临着事实准确性、潜在偏见和计算资源消耗等挑战。`,
    metadata: { source: "技术文档", topic: "AI", id: "3" },
  },
];

async function main() {
  console.log("正在准备RAG系统...");

  // 处理文档
  const documents = [];
  for (const doc of DOCUMENTS) {
    const chunks = await createAndSplitDocument(doc.text, doc.metadata, {
      chunkSize: 200, // 每个块最大字符数
      chunkOverlap: 50, // 块之间的重叠字符数
    });
    documents.push(...chunks);
  }

  console.log(`已处理${documents.length}个文档块`);

  // 创建RAG系统
  const rag = await new RAGBuilder()
    .addDocuments(documents)
    .withLLM({
      provider: ModelProvider.OPENAI,
      modelName: "gpt-3.5-turbo",
      apiKey: API_KEY,
      temperature: 0.3, // 降低随机性，使回答更确定性
    })
    .withRAGConfig({
      embeddingModelName: "text-embedding-ada-002",
      vectorStoreType: "memory",
    })
    .build();

  console.log("RAG系统初始化完成！");
  console.log("-------------------");

  // 示例查询
  const queries = [
    "Node.js有什么特点？",
    "TypeScript和JavaScript的关系是什么？",
    "什么是大语言模型，它有哪些应用？",
  ];

  for (const query of queries) {
    console.log(`问题: ${query}`);

    // 执行查询
    const answer = await rag.query(query);
    console.log(`回答: ${answer}`);
    console.log("-------------------");
  }
}

main().catch((error) => {
  console.error("出错了:", error);
});
