/**
 * Agent构建器模块
 *
 * 用于简化Agent的构建过程
 */

import { ChatOpenAI } from "@langchain/openai";
import { AgentExecutor, createOpenAIToolsAgent } from "langchain/agents";
import {
  ChatPromptTemplate,
  MessagesPlaceholder,
} from "@langchain/core/prompts";
import { BaseMessage } from "@langchain/core/messages";
import { DynamicStructuredTool } from "@langchain/core/tools";
import { LLMConfig } from "../types.js";
import { loadLLM } from "../utils/modelLoader.js";

/**
 * Agent构建器类
 *
 * 用于简化LLM Agent的构建过程
 */
export class AgentBuilder {
  private llmConfig?: LLMConfig;
  private tools: DynamicStructuredTool[] = [];
  private executionHistory: BaseMessage[] = [];
  private prompt?: ChatPromptTemplate;
  private agentExecutor?: AgentExecutor;

  /**
   * 配置Agent的语言模型
   * @param config LLM配置
   * @returns 当前Agent构建器实例
   */
  public withLLM(config: LLMConfig): AgentBuilder {
    this.llmConfig = config;
    return this;
  }

  /**
   * 添加工具到Agent
   * @param tools 工具数组
   * @returns 当前Agent构建器实例
   */
  public withTools(tools: DynamicStructuredTool[]): AgentBuilder {
    this.tools.push(...tools);
    return this;
  }

  /**
   * 添加单个工具到Agent
   * @param tool 工具
   * @returns 当前Agent构建器实例
   */
  public withTool(tool: DynamicStructuredTool): AgentBuilder {
    this.tools.push(tool);
    return this;
  }

  /**
   * 添加执行历史记录，用于支持多轮对话
   * @param messages 历史消息数组
   * @returns 当前Agent构建器实例
   */
  public withExecutionHistory(messages: BaseMessage[]): AgentBuilder {
    this.executionHistory = messages;
    return this;
  }

  /**
   * 自定义Agent提示模板
   * @param prompt 自定义prompt模板
   * @returns 当前Agent构建器实例
   */
  public withCustomPrompt(prompt: ChatPromptTemplate): AgentBuilder {
    this.prompt = prompt;
    return this;
  }

  /**
   * 构建默认的Agent提示模板
   * @returns 提示模板
   */
  private buildDefaultPrompt(): ChatPromptTemplate {
    return ChatPromptTemplate.fromMessages([
      [
        "system",
        `你是一个有用的AI助手，可以使用提供的工具来回答用户的问题。
使用工具时，先思考应该使用哪个工具，然后使用工具回答用户的问题。
如果不需要使用工具，直接用你自己的知识回答。
始终以友好、简洁的方式用中文回复用户。`,
      ],
      new MessagesPlaceholder("chat_history"),
      ["human", "{input}"],
      new MessagesPlaceholder("agent_scratchpad"),
    ]);
  }

  /**
   * 构建Agent
   * @returns 当前Agent构建器实例
   */
  public async build(): Promise<AgentBuilder> {
    if (!this.llmConfig) {
      throw new Error("必须提供LLM配置");
    }

    if (this.tools.length === 0) {
      throw new Error("Agent必须至少有一个工具");
    }

    const llm = loadLLM(this.llmConfig);
    const finalPrompt = this.prompt || this.buildDefaultPrompt();

    const agent = await createOpenAIToolsAgent({
      llm,
      tools: this.tools,
      prompt: finalPrompt,
    });

    this.agentExecutor = new AgentExecutor({
      agent,
      tools: this.tools,
      // 配置执行选项
      returnIntermediateSteps: true, // 返回中间步骤，用于调试
      maxIterations: 5, // 最大执行步骤数
    });

    return this;
  }

  /**
   * 执行Agent查询
   * @param input 用户输入
   * @returns 执行结果
   */
  public async execute(
    input: string
  ): Promise<{ output: string; intermediateSteps: any[] }> {
    if (!this.agentExecutor) {
      throw new Error("必须先调用build()方法构建Agent");
    }

    const result = await this.agentExecutor.invoke({
      input,
      chat_history: this.executionHistory,
    });

    return {
      output: result.output,
      intermediateSteps: result.intermediateSteps || [],
    };
  }
}
