/**
 * Agent工具模块
 *
 * 提供各种预定义的工具，供Agent使用
 */

import { z } from "zod";
import { DynamicStructuredTool } from "@langchain/core/tools";
import { AgentTool } from "../types.js";

/**
 * 创建基础计算器工具
 * @returns 计算器工具
 */
export function createCalculatorTool(): DynamicStructuredTool {
  return new DynamicStructuredTool({
    name: "calculator",
    description: "进行基本的数学计算，如加减乘除",
    schema: z.object({
      expression: z
        .string()
        .describe('需要计算的数学表达式，例如 "2 + 2" 或 "15 * 6"'),
    }),
    func: async ({ expression }) => {
      try {
        // 注意：在实际生产环境中，应当进行更严格的安全检查，防止代码注入
        const safeExpression = expression.replace(/[^0-9+\-*/().\s]/g, "");
        const result = eval(safeExpression);
        return result.toString();
      } catch (error) {
        return `计算错误: ${
          error instanceof Error ? error.message : String(error)
        }`;
      }
    },
  });
}

/**
 * 创建时间工具
 * @returns 时间工具
 */
export function createDateTimeTool(): DynamicStructuredTool {
  return new DynamicStructuredTool({
    name: "datetime",
    description: "获取当前日期和时间信息",
    schema: z.object({
      format: z
        .string()
        .optional()
        .describe('日期格式，例如 "ISO"、"日期"、"时间"或"时间戳"'),
    }),
    func: async ({ format }) => {
      const now = new Date();

      switch (format?.toLowerCase()) {
        case "iso":
          return now.toISOString();
        case "日期":
          return now.toLocaleDateString("zh-CN");
        case "时间":
          return now.toLocaleTimeString("zh-CN");
        case "时间戳":
          return now.getTime().toString();
        default:
          return now.toLocaleString("zh-CN");
      }
    },
  });
}

/**
 * 创建网络搜索工具（模拟）
 * @returns 网络搜索工具
 */
export function createSearchTool(): DynamicStructuredTool {
  return new DynamicStructuredTool({
    name: "search",
    description: "在互联网上搜索信息",
    schema: z.object({
      query: z.string().describe("搜索查询关键词"),
    }),
    func: async ({ query }) => {
      // 注意：这只是一个模拟，实际实现应当调用真实的搜索API
      return `模拟搜索结果: ${query}的搜索结果将显示在这里。在实际应用中，你应该集成真实的搜索API。`;
    },
  });
}

/**
 * 从通用AgentTool接口创建DynamicStructuredTool
 * @param tool 通用工具接口
 * @returns 可用于LangChain Agent的工具
 */
export function createToolFromInterface(
  tool: AgentTool
): DynamicStructuredTool {
  return new DynamicStructuredTool({
    name: tool.name,
    description: tool.description,
    schema: tool.schema
      ? z.object(tool.schema)
      : z.object({ input: z.string() }),
    func: tool.func,
  });
}

/**
 * 创建一组常用工具
 * @returns 常用工具数组
 */
export function createCommonTools(): DynamicStructuredTool[] {
  return [createCalculatorTool(), createDateTimeTool(), createSearchTool()];
}
