/**
 * 聊天构建器模块
 *
 * 用于简化聊天应用的构建过程
 */

import { ChatOpenAI } from "@langchain/openai";
import {
  ChatPromptTemplate,
  SystemMessagePromptTemplate,
  HumanMessagePromptTemplate,
  MessagesPlaceholder,
} from "@langchain/core/prompts";
import {
  BaseMessage,
  HumanMessage,
  AIMessage,
  SystemMessage,
} from "@langchain/core/messages";
import { StringOutputParser } from "@langchain/core/output_parsers";
import { LLMConfig } from "../types.js";
import { loadLLM } from "../utils/modelLoader.js";
import { RunnableSequence } from "@langchain/core/runnables";
import { estimateMessagesTokenCount } from "../utils/tokenCounter.js";

/**
 * 聊天配置接口
 */
export interface ChatConfig {
  /** 系统提示文本 */
  systemPrompt?: string;
  /** 记忆消息的最大数量 */
  memoryMessageLimit?: number;
  /** 记忆消息的最大token数量 */
  memoryTokenLimit?: number;
  /** 流式响应开关 */
  streamingEnabled?: boolean;
}

/**
 * 聊天构建器类
 *
 * 用于构建聊天应用
 */
export class ChatBuilder {
  private llmConfig?: LLMConfig;
  private chatConfig: ChatConfig = {
    systemPrompt: "你是一个有帮助的AI助手。",
    memoryMessageLimit: 10,
    memoryTokenLimit: 2000,
    streamingEnabled: false,
  };
  private chatHistory: BaseMessage[] = [];
  private llm?: ChatOpenAI;
  private chain?: RunnableSequence;

  /**
   * 配置聊天的语言模型
   * @param config LLM配置
   * @returns 当前聊天构建器实例
   */
  public withLLM(config: LLMConfig): ChatBuilder {
    this.llmConfig = config;
    return this;
  }

  /**
   * 配置聊天参数
   * @param config 聊天配置
   * @returns 当前聊天构建器实例
   */
  public withConfig(config: Partial<ChatConfig>): ChatBuilder {
    this.chatConfig = { ...this.chatConfig, ...config };
    return this;
  }

  /**
   * 添加聊天历史
   * @param messages 聊天消息数组
   * @returns 当前聊天构建器实例
   */
  public withHistory(messages: BaseMessage[]): ChatBuilder {
    this.chatHistory = messages;
    return this;
  }

  /**
   * 设置系统提示
   * @param systemPrompt 系统提示文本
   * @returns 当前聊天构建器实例
   */
  public withSystemPrompt(systemPrompt: string): ChatBuilder {
    this.chatConfig.systemPrompt = systemPrompt;
    return this;
  }

  /**
   * 构建聊天应用
   * @returns 当前聊天构建器实例
   */
  public build(): ChatBuilder {
    if (!this.llmConfig) {
      throw new Error("必须提供LLM配置");
    }

    this.llm = loadLLM(this.llmConfig);

    const prompt = ChatPromptTemplate.fromMessages([
      SystemMessagePromptTemplate.fromTemplate(
        this.chatConfig.systemPrompt || ""
      ),
      new MessagesPlaceholder("history"),
      HumanMessagePromptTemplate.fromTemplate("{input}"),
    ]);

    this.chain = RunnableSequence.from([
      {
        input: (input: string) => input,
        history: () => this.chatHistory,
      },
      prompt,
      this.llm,
      new StringOutputParser(),
    ]);

    return this;
  }

  /**
   * 修剪历史记录以符合token限制
   * @private
   */
  private trimHistory(): void {
    // 如果未设置限制，直接返回
    if (
      !this.chatConfig.memoryMessageLimit &&
      !this.chatConfig.memoryTokenLimit
    ) {
      return;
    }

    // 按消息数量限制
    if (
      this.chatConfig.memoryMessageLimit &&
      this.chatHistory.length > this.chatConfig.memoryMessageLimit
    ) {
      // 保留系统消息
      const systemMessages = this.chatHistory.filter(
        (msg) => msg instanceof SystemMessage
      );
      const nonSystemMessages = this.chatHistory.filter(
        (msg) => !(msg instanceof SystemMessage)
      );

      // 保留最新的消息
      const keptMessages = nonSystemMessages.slice(
        -this.chatConfig.memoryMessageLimit
      );
      this.chatHistory = [...systemMessages, ...keptMessages];
    }

    // 按token数量限制
    if (this.chatConfig.memoryTokenLimit) {
      let totalTokens = estimateMessagesTokenCount(this.chatHistory);

      // 如果超出限制，从最旧的非系统消息开始删除
      if (totalTokens > this.chatConfig.memoryTokenLimit) {
        // 保留系统消息
        const systemMessages = this.chatHistory.filter(
          (msg) => msg instanceof SystemMessage
        );
        let nonSystemMessages = this.chatHistory.filter(
          (msg) => !(msg instanceof SystemMessage)
        );

        while (nonSystemMessages.length > 0) {
          // 移除最旧的消息
          nonSystemMessages.shift();

          // 重新计算token数
          const newHistory = [...systemMessages, ...nonSystemMessages];
          totalTokens = estimateMessagesTokenCount(newHistory);

          // 如果token数符合限制，更新历史记录并退出循环
          if (totalTokens <= this.chatConfig.memoryTokenLimit) {
            this.chatHistory = newHistory;
            break;
          }
        }
      }
    }
  }

  /**
   * 向聊天发送消息并获取回复
   * @param message 用户消息
   * @returns 助手回复
   */
  public async sendMessage(message: string): Promise<string> {
    if (!this.chain) {
      throw new Error("必须先调用build()方法构建聊天");
    }

    // 添加用户消息到历史
    this.chatHistory.push(new HumanMessage(message));

    // 修剪历史记录
    this.trimHistory();

    // 生成回复
    let response: string;

    if (this.chatConfig.streamingEnabled) {
      // 收集流式响应片段
      const chunks: string[] = [];
      for await (const chunk of await this.chain.stream(message)) {
        chunks.push(chunk);
      }
      response = chunks.join("");
    } else {
      response = await this.chain.invoke(message);
    }

    // 添加AI回复到历史
    this.chatHistory.push(new AIMessage(response));

    return response;
  }

  /**
   * 向聊天发送消息并获取流式回复
   * @param message 用户消息
   * @returns 助手回复流
   */
  public async streamMessage(message: string): Promise<AsyncIterable<string>> {
    if (!this.chain) {
      throw new Error("必须先调用build()方法构建聊天");
    }

    // 添加用户消息到历史
    this.chatHistory.push(new HumanMessage(message));

    // 修剪历史记录
    this.trimHistory();

    // 返回流式响应
    const stream = await this.chain.stream(message);

    // 由于我们不能等待流完成来添加到历史，所以这里需要收集完整响应
    const responseCollector = this.collectStreamResponse(stream);

    return responseCollector.stream;
  }

  /**
   * 收集流式响应并添加到历史
   * @private
   * @param stream 原始响应流
   * @returns 打包的响应流和完整响应Promise
   */
  private collectStreamResponse(stream: AsyncIterable<string>): {
    stream: AsyncIterable<string>;
    fullResponsePromise: Promise<string>;
  } {
    const chunks: string[] = [];

    // 创建一个异步生成器
    const newStream = (async function* () {
      for await (const chunk of stream) {
        chunks.push(chunk);
        yield chunk;
      }
    })();

    // 创建一个Promise来处理完整响应
    const fullResponsePromise = (async () => {
      const fullResponse = chunks.join("");
      // 添加AI回复到历史
      this.chatHistory.push(new AIMessage(fullResponse));
      return fullResponse;
    })();

    return {
      stream: newStream,
      fullResponsePromise,
    };
  }

  /**
   * 清空聊天历史
   * @returns 当前聊天构建器实例
   */
  public clearHistory(): ChatBuilder {
    // 保留系统消息
    const systemMessages = this.chatHistory.filter(
      (msg) => msg instanceof SystemMessage
    );
    this.chatHistory = systemMessages;
    return this;
  }

  /**
   * 获取聊天历史
   * @returns 聊天历史消息数组
   */
  public getHistory(): BaseMessage[] {
    return [...this.chatHistory];
  }
}
