/**
 * RAG构建器模块
 *
 * 用于简化RAG应用的构建过程
 */

import { Document } from "@langchain/core/documents";
import { MemoryVectorStore } from "langchain/vectorstores/memory";
import { OpenAIEmbeddings } from "@langchain/openai";
import { ChatOpenAI } from "@langchain/openai";
import { loadLLM, loadEmbeddingModel } from "../utils/modelLoader.js";
import { RAGConfig, LLMConfig, ModelProvider } from "../types.js";
import {
  RunnableSequence,
  RunnablePassthrough,
} from "@langchain/core/runnables";
import { StringOutputParser } from "@langchain/core/output_parsers";
import { formatDocumentsAsString } from "langchain/util/document";
import {
  ChatPromptTemplate,
  SystemMessagePromptTemplate,
  HumanMessagePromptTemplate,
} from "@langchain/core/prompts";

/**
 * RAG构建器类
 *
 * 用于构建检索增强生成系统
 */
export class RAGBuilder {
  private documents: Document[] = [];
  private vectorStore: MemoryVectorStore | null = null;
  private llmConfig?: LLMConfig;
  private ragConfig?: RAGConfig;
  private chain: RunnableSequence | null = null;

  /**
   * 添加文档到RAG系统
   * @param docs 文档数组
   * @returns 当前RAG构建器实例
   */
  public addDocuments(docs: Document[]): RAGBuilder {
    this.documents.push(...docs);
    return this;
  }

  /**
   * 配置大语言模型
   * @param config LLM配置
   * @returns 当前RAG构建器实例
   */
  public withLLM(config: LLMConfig): RAGBuilder {
    this.llmConfig = config;
    return this;
  }

  /**
   * 配置RAG系统参数
   * @param config RAG配置
   * @returns 当前RAG构建器实例
   */
  public withRAGConfig(config: RAGConfig): RAGBuilder {
    this.ragConfig = config;
    return this;
  }

  /**
   * 构建RAG系统
   * @returns 当前RAG构建器实例
   */
  public async build(): Promise<RAGBuilder> {
    if (!this.llmConfig) {
      throw new Error("必须提供LLM配置");
    }

    if (!this.ragConfig) {
      throw new Error("必须提供RAG配置");
    }

    if (this.documents.length === 0) {
      throw new Error("必须添加至少一个文档");
    }

    // 加载嵌入模型
    const embeddings = loadEmbeddingModel(
      this.llmConfig.provider,
      this.llmConfig.apiKey,
      this.ragConfig.embeddingModelName
    );

    // 创建向量存储
    this.vectorStore = await MemoryVectorStore.fromDocuments(
      this.documents,
      embeddings
    );

    // 创建检索器
    const retriever = this.vectorStore.asRetriever();

    // 加载LLM模型
    const llm = loadLLM(this.llmConfig);

    // 创建提示模板
    const prompt = ChatPromptTemplate.fromMessages([
      SystemMessagePromptTemplate.fromTemplate(
        "你是一个有帮助的助手，负责基于以下上下文回答用户问题。如果问题不在上下文中，请说你不知道，不要编造答案。"
      ),
      HumanMessagePromptTemplate.fromTemplate(
        "上下文信息:\n\n{context}\n\n用户问题: {question}"
      ),
    ]);

    // 创建RAG链
    this.chain = RunnableSequence.from([
      {
        context: retriever.pipe(formatDocumentsAsString),
        question: new RunnablePassthrough(),
      },
      prompt,
      llm,
      new StringOutputParser(),
    ]);

    return this;
  }

  /**
   * 查询RAG系统
   * @param query 查询字符串
   * @returns 查询结果
   */
  public async query(query: string): Promise<string> {
    if (!this.chain) {
      throw new Error("必须先调用build()方法构建RAG系统");
    }

    return await this.chain.invoke(query);
  }

  /**
   * 查询RAG系统(流式响应)
   * @param query 查询字符串
   * @returns 查询结果流
   */
  public async streamingQuery(query: string): Promise<AsyncIterable<string>> {
    if (!this.chain) {
      throw new Error("必须先调用build()方法构建RAG系统");
    }

    return await this.chain.stream(query);
  }
}
