/**
 * 大模型健康检查工具
 *
 * 用于验证API连接和模型可用性
 */

import { ChatOpenAI } from "@langchain/openai";
import { ModelProvider, LLMConfig } from "../types.js";
import { loadLLM } from "./modelLoader.js";

/**
 * 健康检查结果
 */
export interface HealthCheckResult {
  /** 是否通过检查 */
  success: boolean;
  /** 延迟（毫秒） */
  latencyMs?: number;
  /** 错误信息 */
  error?: string;
  /** 额外信息 */
  info?: Record<string, any>;
  /** 建议（如有） */
  suggestion?: string;
}

/**
 * 执行LLM模型健康检查
 * @param config LLM配置
 * @returns 健康检查结果
 */
export async function checkLLMHealth(
  config: LLMConfig
): Promise<HealthCheckResult> {
  const startTime = Date.now();

  try {
    const llm = loadLLM({
      ...config,
      modelName: config.modelName || "gpt-3.5-turbo", // 确保有一个模型名
    });

    // 执行一个简单的查询来测试连接
    const testPrompt = "回复数字1";
    const result = await llm.invoke(testPrompt);
    const content =
      typeof result.content === "string"
        ? result.content
        : JSON.stringify(result.content);

    const endTime = Date.now();
    const latencyMs = endTime - startTime;

    // 验证响应是否合理
    const isValidResponse = content.includes("1");

    if (isValidResponse) {
      return {
        success: true,
        latencyMs,
        info: {
          modelName: config.modelName,
          provider: config.provider,
          response: content,
        },
      };
    } else {
      return {
        success: false,
        latencyMs,
        error: "模型响应不符合预期",
        info: {
          modelName: config.modelName,
          provider: config.provider,
          response: content,
        },
        suggestion: "请检查模型配置是否正确",
      };
    }
  } catch (error) {
    const endTime = Date.now();
    const latencyMs = endTime - startTime;

    return {
      success: false,
      latencyMs,
      error: error instanceof Error ? error.message : String(error),
      info: {
        modelName: config.modelName,
        provider: config.provider,
      },
      suggestion: getErrorSuggestion(error, config.provider),
    };
  }
}

/**
 * 执行嵌入模型健康检查
 * @param provider 模型提供商
 * @param apiKey API密钥
 * @param modelName 模型名称
 * @returns 健康检查结果
 */
export async function checkEmbeddingHealth(
  provider: ModelProvider,
  apiKey: string,
  modelName: string = "text-embedding-ada-002"
): Promise<HealthCheckResult> {
  const startTime = Date.now();

  try {
    const { loadEmbeddingModel } = await import("./modelLoader.js");
    const embeddings = loadEmbeddingModel(provider, apiKey, modelName);

    // 执行一个简单的嵌入操作来测试连接
    const testText = "测试文本";
    const embeddingResult = await embeddings.embedQuery(testText);

    const endTime = Date.now();
    const latencyMs = endTime - startTime;

    // 验证嵌入向量是否合理
    const isValidEmbedding =
      Array.isArray(embeddingResult) &&
      embeddingResult.length > 0 &&
      typeof embeddingResult[0] === "number";

    if (isValidEmbedding) {
      return {
        success: true,
        latencyMs,
        info: {
          modelName,
          provider,
          dimensions: embeddingResult.length,
        },
      };
    } else {
      return {
        success: false,
        latencyMs,
        error: "嵌入模型响应不符合预期",
        info: {
          modelName,
          provider,
        },
        suggestion: "请检查嵌入模型配置是否正确",
      };
    }
  } catch (error) {
    const endTime = Date.now();
    const latencyMs = endTime - startTime;

    return {
      success: false,
      latencyMs,
      error: error instanceof Error ? error.message : String(error),
      info: {
        modelName,
        provider,
      },
      suggestion: getErrorSuggestion(error, provider),
    };
  }
}

/**
 * 根据错误类型提供建议
 * @param error 错误对象
 * @param provider 模型提供商
 * @returns 建议文本
 */
function getErrorSuggestion(error: unknown, provider: ModelProvider): string {
  const errorMessage = error instanceof Error ? error.message : String(error);

  if (errorMessage.includes("API key")) {
    return "API密钥似乎无效，请检查您的API密钥配置";
  }

  if (errorMessage.includes("timeout") || errorMessage.includes("ETIMEDOUT")) {
    return "连接超时，请检查您的网络连接或代理设置";
  }

  if (errorMessage.includes("rate limit")) {
    return "触发了速率限制，请稍后再试或减少请求频率";
  }

  switch (provider) {
    case ModelProvider.OPENAI:
      return "连接OpenAI API时出错，请确保您的API密钥有效且网络正常";
    case ModelProvider.AZURE_OPENAI:
      return "连接Azure OpenAI API时出错，请检查您的端点URL和API密钥";
    case ModelProvider.ANTHROPIC:
      return "连接Anthropic API时出错，请检查您的API密钥";
    case ModelProvider.LOCAL:
      return "连接本地模型时出错，请确保模型已正确加载";
    default:
      return "连接API时出错，请检查您的配置";
  }
}
