/**
 * 模型检测工具
 *
 * 用于检测环境中可用的大模型和嵌入模型
 */

import { ModelProvider } from "../types.js";

/**
 * 模型可用性检测结果
 */
export interface ModelDetectionResult {
  /** 是否可用 */
  available: boolean;
  /** 详细信息 */
  details?: string;
  /** 检测到的token限制 */
  tokenLimit?: number;
}

/**
 * 检测OpenAI模型是否可用
 * @param apiKey OpenAI API密钥
 * @returns 检测结果
 */
export async function detectOpenAIModels(
  apiKey: string
): Promise<Record<string, ModelDetectionResult>> {
  if (!apiKey) {
    return {
      "gpt-3.5-turbo": {
        available: false,
        details: "未提供API密钥",
      },
      "gpt-4": {
        available: false,
        details: "未提供API密钥",
      },
    };
  }

  try {
    // 这里应该实际调用OpenAI API的模型列表接口
    // 由于这只是一个示例，我们返回模拟数据
    return {
      "gpt-3.5-turbo": {
        available: true,
        tokenLimit: 4096,
      },
      "gpt-3.5-turbo-16k": {
        available: true,
        tokenLimit: 16384,
      },
      "gpt-4": {
        available: true,
        tokenLimit: 8192,
      },
      "gpt-4-turbo": {
        available: true,
        tokenLimit: 128000,
      },
    };
  } catch (error) {
    return {
      "gpt-3.5-turbo": {
        available: false,
        details: `API错误: ${
          error instanceof Error ? error.message : String(error)
        }`,
      },
      "gpt-4": {
        available: false,
        details: `API错误: ${
          error instanceof Error ? error.message : String(error)
        }`,
      },
    };
  }
}

/**
 * 检测本地模型是否可用
 * @returns 检测结果
 */
export async function detectLocalModels(): Promise<
  Record<string, ModelDetectionResult>
> {
  // 这是一个简化的示例，实际应该检测本地可用的模型
  return {
    "llama-2-7b": {
      available: false,
      details: "未找到本地模型",
    },
    "llama-2-13b": {
      available: false,
      details: "未找到本地模型",
    },
  };
}

/**
 * 检测所有可用的模型
 * @param providers 要检测的提供商列表
 * @param apiKeys API密钥映射
 * @returns 检测结果
 */
export async function detectAvailableModels(
  providers: ModelProvider[] = [ModelProvider.OPENAI, ModelProvider.LOCAL],
  apiKeys: Record<string, string> = {}
): Promise<Record<ModelProvider, Record<string, ModelDetectionResult>>> {
  const results: Record<ModelProvider, Record<string, ModelDetectionResult>> = {
    [ModelProvider.OPENAI]: {},
    [ModelProvider.AZURE_OPENAI]: {},
    [ModelProvider.ANTHROPIC]: {},
    [ModelProvider.LOCAL]: {},
  };

  // 并行检测所有提供商
  const promises = providers.map(async (provider) => {
    switch (provider) {
      case ModelProvider.OPENAI:
        results[provider] = await detectOpenAIModels(apiKeys[provider] || "");
        break;
      case ModelProvider.LOCAL:
        results[provider] = await detectLocalModels();
        break;
      // 其他提供商的检测可以在这里添加
      default:
        results[provider] = {};
    }
  });

  await Promise.all(promises);
  return results;
}
