/**
 * 模型加载器工具类
 *
 * 提供加载不同类型LLM模型和嵌入模型的工具函数
 */

import { ChatOpenAI } from "@langchain/openai";
import { OpenAIEmbeddings } from "@langchain/openai";
import { LLMConfig, ModelProvider } from "../types.js";

/**
 * 加载LLM模型
 * @param config 模型配置
 * @returns 返回LLM模型实例
 */
export function loadLLM(config: LLMConfig) {
  const { provider, modelName, apiKey, temperature = 0.7, ...rest } = config;

  switch (provider) {
    case ModelProvider.OPENAI:
      return new ChatOpenAI({
        modelName,
        openAIApiKey: apiKey,
        temperature,
        ...rest,
      });
    case ModelProvider.AZURE_OPENAI:
      // Azure OpenAI 需要单独设置配置，请按照实际 SDK 参数要求设置
      return new ChatOpenAI({
        modelName,
        openAIApiKey: apiKey,
        temperature,
        ...rest,
      });
    // 可以继续添加其他模型提供商的支持
    default:
      throw new Error(`不支持的模型提供商: ${provider}`);
  }
}

/**
 * 加载嵌入模型
 * @param provider 模型提供商
 * @param apiKey API密钥
 * @param modelName 模型名称
 * @returns 返回嵌入模型实例
 */
export function loadEmbeddingModel(
  provider: ModelProvider,
  apiKey: string,
  modelName: string = "text-embedding-ada-002"
) {
  switch (provider) {
    case ModelProvider.OPENAI:
      return new OpenAIEmbeddings({
        openAIApiKey: apiKey,
        modelName,
      });
    case ModelProvider.AZURE_OPENAI:
      // Azure OpenAI 需要单独设置配置，请按照实际 SDK 参数要求设置
      return new OpenAIEmbeddings({
        openAIApiKey: apiKey,
        modelName,
      });
    // 可以继续添加其他嵌入模型的支持
    default:
      throw new Error(`不支持的嵌入模型提供商: ${provider}`);
  }
}
