/**
 * 提示模板构建工具
 *
 * 用于简化提示模板的创建过程
 */

import {
  ChatPromptTemplate,
  SystemMessagePromptTemplate,
  HumanMessagePromptTemplate,
  AIMessagePromptTemplate,
  MessagesPlaceholder,
} from "@langchain/core/prompts";
import { BaseMessage } from "@langchain/core/messages";

/**
 * 提示模板构建器
 *
 * 简化ChatPromptTemplate的创建过程
 */
export class PromptBuilder {
  private messages: any[] = [];

  /**
   * 添加系统提示
   * @param text 系统提示文本
   * @returns 当前构建器实例
   */
  public addSystemMessage(text: string): PromptBuilder {
    this.messages.push(SystemMessagePromptTemplate.fromTemplate(text));
    return this;
  }

  /**
   * 添加人类提示
   * @param text 人类提示模板文本
   * @returns 当前构建器实例
   */
  public addHumanMessage(text: string): PromptBuilder {
    this.messages.push(HumanMessagePromptTemplate.fromTemplate(text));
    return this;
  }

  /**
   * 添加AI提示
   * @param text AI提示模板文本
   * @returns 当前构建器实例
   */
  public addAIMessage(text: string): PromptBuilder {
    this.messages.push(AIMessagePromptTemplate.fromTemplate(text));
    return this;
  }

  /**
   * 添加消息占位符
   * @param variableName 变量名称
   * @returns 当前构建器实例
   */
  public addMessagesPlaceholder(variableName: string): PromptBuilder {
    this.messages.push(new MessagesPlaceholder(variableName));
    return this;
  }

  /**
   * 添加原始消息
   * @param message 原始消息对象
   * @returns 当前构建器实例
   */
  public addMessage(message: any): PromptBuilder {
    this.messages.push(message);
    return this;
  }

  /**
   * 添加示例对话
   * @param humanText 人类消息
   * @param aiText AI回复
   * @returns 当前构建器实例
   */
  public addExampleDialog(humanText: string, aiText: string): PromptBuilder {
    this.messages.push(HumanMessagePromptTemplate.fromTemplate(humanText));
    this.messages.push(AIMessagePromptTemplate.fromTemplate(aiText));
    return this;
  }

  /**
   * 构建聊天提示模板
   * @returns 聊天提示模板
   */
  public build(): ChatPromptTemplate {
    return ChatPromptTemplate.fromMessages(this.messages);
  }
}

/**
 * 创建RAG提示模板
 * @param systemPrompt 系统提示（可选）
 * @returns 聊天提示模板
 */
export function createRAGPromptTemplate(
  systemPrompt?: string
): ChatPromptTemplate {
  return new PromptBuilder()
    .addSystemMessage(
      systemPrompt ||
        "你是一个有帮助的助手，负责基于提供的上下文回答用户问题。如果问题不在上下文中，请说不知道，不要编造答案。"
    )
    .addHumanMessage("上下文信息:\n\n{context}\n\n用户问题: {question}")
    .build();
}

/**
 * 创建Agent提示模板
 * @param systemPrompt 系统提示（可选）
 * @returns 聊天提示模板
 */
export function createAgentPromptTemplate(
  systemPrompt?: string
): ChatPromptTemplate {
  return new PromptBuilder()
    .addSystemMessage(
      systemPrompt ||
        `你是一个有用的AI助手，可以使用提供的工具来回答用户的问题。
使用工具时，先思考应该使用哪个工具，然后使用工具回答用户的问题。
如果不需要使用工具，直接用你自己的知识回答。
始终以友好、简洁的方式用中文回复用户。`
    )
    .addMessagesPlaceholder("chat_history")
    .addHumanMessage("{input}")
    .addMessagesPlaceholder("agent_scratchpad")
    .build();
}

/**
 * 创建聊天提示模板
 * @param systemPrompt 系统提示（可选）
 * @returns 聊天提示模板
 */
export function createChatPromptTemplate(
  systemPrompt?: string
): ChatPromptTemplate {
  return new PromptBuilder()
    .addSystemMessage(systemPrompt || "你是一个有帮助的AI助手。")
    .addMessagesPlaceholder("history")
    .addHumanMessage("{input}")
    .build();
}

/**
 * 创建翻译提示模板
 * @param targetLanguage 目标语言
 * @returns 聊天提示模板
 */
export function createTranslationPromptTemplate(
  targetLanguage: string
): ChatPromptTemplate {
  return new PromptBuilder()
    .addSystemMessage(
      `你是一个专业的翻译助手，擅长将文本翻译成${targetLanguage}。
保持原文的意思、风格和格式，但使用地道的${targetLanguage}表达。
如果遇到专业术语，确保使用对应领域在${targetLanguage}中的标准翻译。`
    )
    .addHumanMessage("请将以下文本翻译成${targetLanguage}:\n\n{text}")
    .build();
}

/**
 * 创建内容提取提示模板
 * @returns 聊天提示模板
 */
export function createExtractionPromptTemplate(): ChatPromptTemplate {
  return new PromptBuilder()
    .addSystemMessage(
      `你是一个专业的信息提取助手，擅长从文本中提取结构化信息。
请从给定文本中提取所需的信息，并按照指定的格式输出。
输出应该是一个有效的JSON对象，符合给定的schema。`
    )
    .addHumanMessage(
      `请从以下文本中提取信息:\n\n{text}\n\n
需要提取的信息格式为: {schema}\n
请输出JSON格式的结果。`
    )
    .build();
}
